==> catequil-webshop-1.0.11-758356.tar: src/Model/WebshopOrder.php <==
<?php
declare(strict_types=1);

namespace Catequil\Webshop\Model;

use Catequil\Webshop\Repository\WebshopOrderRepository;
use DateTime;
use Doctrine\DBAL\Types\Types;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\GeneratedValue;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\JoinColumn;
use Doctrine\ORM\Mapping\ManyToOne;
use Doctrine\ORM\Mapping\Table;
use InvalidArgumentException;
use JetBrains\PhpStorm\ArrayShape;
use function array_fill;
use function array_filter;
use function array_map;
use function array_merge;
use function array_values;
use function count;
use function implode;
use function preg_replace;
use function sprintf;

/**
 * Class WebshopOrder
 *
 * @package Catequil\Webshop\Model
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-webshop
 */
#[Entity(WebshopOrderRepository::class)]
#[Table('webshop_order', options: ['collate' => 'utf8mb4_croatian_ci'])]
class WebshopOrder implements WebshopOrderInterface
{
    //<editor-fold desc="Entity Fields">
    #[Id]
    #[GeneratedValue('AUTO')]
    #[Column(type: Types::BIGINT, options: ['unsigned' => true])]
    protected int $internalId;
    #[ManyToOne(WebshopInterface::class)]
    #[JoinColumn('webshop_id', 'id', nullable: false)]
    protected WebshopInterface $webshop;
    #[Column(type: Types::BIGINT, options: ['unsigned' => true])]
    protected int $id;
    #[Column(type: Types::DATETIME_MUTABLE)]
    protected DateTime $updatedAt;
    #[Column(type: Types::STRING, length: 191)]
    protected string $status;
    #[Column(type: Types::STRING, length: 191)]
    protected string $shippingMethod;

    //</editor-fold>

    #[ArrayShape([
        'ID' => 'int',
        'post_status' => 'string',
    ])]
    private ?array $wooOrder = null;
    #[ArrayShape([[
        'order_item_id' => 'int',
    ]])]
    private ?array $wooOrderItems = null;
    private ?array $wooOrderItemMeta = null;
    #[ArrayShape([

    ])]
    private ?array $wooOrderMeta = null;

    /**
     * @param WebshopInterface $webshop
     * @param int $id
     */
    public function __construct(WebshopInterface $webshop, int $id)
    {
        $this->webshop = $webshop;
        $this->id = $id;
        $this->updatedAt = new DateTime();
    }

    //<editor-fold desc="Getters & Setters">

    public function getWebshop(): WebshopInterface
    {
        return $this->webshop;
    }

    public function getId(): int
    {
        return $this->id;
    }

    public function getDetail(string $detail, mixed $default = null): mixed
    {
        return $this->getDetails()[$detail] ?? $default;
    }

    public function getDetails(): array
    {
        return $this->getWooOrderMeta();
    }

    public function getItems(string $type = 'line_item'): array
    {
        return array_values(
            array_filter(
                array_map(
                    fn (array $item) => array_merge($item, [
                        'details' => $this->getWooOrderItemMeta($item['order_item_id']),
                    ]),
                    $this->getWooOrderItems()
                ),
                fn (array $item) => $item['order_item_type'] === $type
            )
        );
    }

    public function getShippingMethod(): string
    {
        return $this->shippingMethod;
    }

    public function getStatus(): string
    {
        return $this->status;
    }

    public function getUpdatedAt(): DateTime
    {
        return $this->updatedAt;
    }

    //</editor-fold>
    //<editor-fold desc="Actions">

    public function update(): void
    {
        $order = $this->getWooOrder();

        $this->status = $order['post_status'];
        $this->updatedAt = new DateTime('now');

        if ($shippingItem = ($this->getItems('shipping')[0] ?? null)) {
            $this->shippingMethod = preg_replace('/:.+$/', '', $shippingItem['details']['method_id']);
        }
    }

    //</editor-fold>
    //<editor-fold desc="WooCommerce Data">

    private function getWooOrder(): array
    {
        if ($this->wooOrder === null) {
            $conn = $this->webshop->getConnection();
            $sql = 'SELECT ID, post_status FROM wp_posts WHERE ID = ?';

            /** @noinspection PhpUnhandledExceptionInspection */
            $this->wooOrder = $conn->executeQuery($sql, [$this->id])
                ->fetchAssociative() ?: null;

            if ($this->wooOrder === null) {
                throw new InvalidArgumentException(
                    sprintf(
                        'Nepostojeća narudžba %d na webshopu %s.',
                        $this->id,
                        $this->webshop->getDomain()
                    )
                );
            }
        }
        return $this->wooOrder;
    }

    private function getWooOrderItems(): array
    {
        if ($this->wooOrderItems === null) {
            $conn = $this->webshop->getConnection();
            $sql = 'SELECT * FROM wp_woocommerce_order_items WHERE order_id = ?';

            /** @noinspection PhpUnhandledExceptionInspection */
            $this->wooOrderItems = $conn->executeQuery($sql, [$this->id])
                ->fetchAllAssociative();
        }
        return $this->wooOrderItems;
    }

    private function getWooOrderItemMeta(?int $id = null): array
    {
        if ($id !== null) {
            return $this->getWooOrderItemMeta()[$id];
        }

        if ($this->wooOrderItemMeta === null) {
            $conn = $this->webshop->getConnection();
            $sql = sprintf(
                'SELECT order_item_id, meta_key, meta_value FROM wp_woocommerce_order_itemmeta WHERE order_item_id IN(%s)',
                implode(',', array_fill(0, count($this->getWooOrderItems()), '?')),
            );

            /** @noinspection PhpUnhandledExceptionInspection */
            $result = $conn->executeQuery($sql,
                array_map(
                    fn (array $item) => $item['order_item_id'],
                    $this->getWooOrderItems(),
                ),
            )->fetchAllAssociative();

            $this->wooOrderItemMeta = [];

            foreach ($result as $meta) {
                $this->wooOrderItemMeta[$meta['order_item_id']][$meta['meta_key']] = $meta['meta_value'];
            }
        }
        return $this->wooOrderItemMeta;
    }

    private function getWooOrderMeta(): array
    {
        if ($this->wooOrderMeta === null) {
            $conn = $this->webshop->getConnection();
            $sql = 'SELECT meta_key, meta_value FROM wp_postmeta WHERE post_id = ?';

            /** @noinspection PhpUnhandledExceptionInspection */
            $this->wooOrderMeta = $conn->executeQuery($sql, [$this->id])
                ->fetchAllKeyValue();
        }
        return $this->wooOrderMeta;
    }

    //</editor-fold>
}

==> catequil-webshop-1.0.21-171352.tar: src/Model/WebshopOrder.php <==
<?php
declare(strict_types=1);

namespace Catequil\Webshop\Model;

use Catequil\Webshop\Repository\WebshopOrderRepository;
use DateTime;
use Doctrine\DBAL\Types\Types;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\HasLifecycleCallbacks;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\JoinColumn;
use Doctrine\ORM\Mapping\ManyToOne;
use Doctrine\ORM\Mapping\PrePersist;
use Doctrine\ORM\Mapping\PreUpdate;
use Doctrine\ORM\Mapping\Table;
use Gath\Product\ProductGateway;
use InvalidArgumentException;
use JetBrains\PhpStorm\ArrayShape;
use function array_fill;
use function array_filter;
use function array_keys;
use function array_map;
use function array_merge;
use function array_values;
use function count;
use function implode;
use function intval;
use function is_int;
use function preg_replace;
use function sprintf;

/**
 * Class WebshopOrder
 *
 * @package Catequil\Webshop\Model
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-webshop
 */
#[Entity(WebshopOrderRepository::class)]
#[Table('webshop_order', options: ['collate' => 'utf8mb4_croatian_ci'])]
#[HasLifecycleCallbacks]
class WebshopOrder implements WebshopOrderInterface
{
    //<editor-fold desc="Entity Fields">
    #[Id]
    #[ManyToOne(WebshopInterface::class)]
    #[JoinColumn('webshop_id', 'id', nullable: false)]
    protected WebshopInterface $webshop;
    #[Id]
    #[Column(type: Types::BIGINT, options: ['unsigned' => true])]
    protected int $id;
    #[Column(type: Types::DATETIME_MUTABLE)]
    protected DateTime $updatedAt;
    #[Column(type: Types::STRING, length: 191)]
    protected string $status;
    #[Column(type: Types::STRING, length: 191)]
    protected string $shippingMethod;

    //</editor-fold>

    #[ArrayShape([
        'ID' => 'int',
        'post_status' => 'string',
    ])]
    private ?array $wooOrder = null;
    #[ArrayShape([[
        'order_item_id' => 'int',
    ]])]
    private ?array $wooOrderItems = null;
    private ?array $wooOrderItemMeta = null;
    #[ArrayShape([

    ])]
    private ?array $wooOrderMeta = null;

    /**
     * @param WebshopInterface $webshop
     * @param int $id
     */
    public function __construct(WebshopInterface $webshop, int $id)
    {
        $this->webshop = $webshop;
        $this->id = $id;
        $this->updatedAt = new DateTime();
    }

    //<editor-fold desc="Getters & Setters">

    public function getWebshop(): WebshopInterface
    {
        return $this->webshop;
    }

    public function getId(): int
    {
        return $this->id;
    }

    public function getDetail(string $detail, mixed $default = null): mixed
    {
        return $this->getDetails()[$detail] ?? $default;
    }

    public function getDetails(): array
    {
        return $this->getWooOrderMeta();
    }

    public function getItems(string $type = 'line_item'): array
    {
        return array_values(
            array_filter(
                array_map(
                    fn (array $item) => array_merge($item, [
                        'details' => $this->getWooOrderItemMeta($item['order_item_id']),
                    ]),
                    $this->getWooOrderItems()
                ),
                fn (array $item) => $item['order_item_type'] === $type
            )
        );
    }

    public function getShippingMethod(): string
    {
        return $this->shippingMethod;
    }

    public function getStatus(): string
    {
        return $this->status;
    }

    public function getUpdatedAt(): DateTime
    {
        return $this->updatedAt;
    }

    public function getUrl(): string
    {
        return sprintf(
            'https://%s/wp/wp-admin/post.php?post=%d&action=edit',
            $this->webshop->getDomain(),
            $this->getId()
        );
    }

    //</editor-fold>
    //<editor-fold desc="Actions">

    #[PrePersist]
    #[PreUpdate]
    public function update(): void
    {
        $order = $this->getWooOrder();

        $this->status = $order['post_status'];
        $this->updatedAt = new DateTime('now');

        if ($shippingItem = ($this->getItems('shipping')[0] ?? null)) {
            $this->shippingMethod = preg_replace('/:.+$/', '', $shippingItem['details']['method_id']);
        }
    }

    //</editor-fold>
    //<editor-fold desc="WooCommerce Data">

    private function getWooOrder(): array
    {
        if ($this->wooOrder === null) {
            $conn = $this->webshop->getConnection();
            $sql = 'SELECT ID, post_status FROM wp_posts WHERE ID = ?';

            /** @noinspection PhpUnhandledExceptionInspection */
            $this->wooOrder = $conn->executeQuery($sql, [$this->id])
                ->fetchAssociative() ?: null;

            if ($this->wooOrder === null) {
                throw new InvalidArgumentException(
                    sprintf(
                        'Nepostojeća narudžba %d na webshopu %s.',
                        $this->id,
                        $this->webshop->getDomain()
                    )
                );
            }
        }
        return $this->wooOrder;
    }

    private function getWooOrderItems(): array
    {
        if ($this->wooOrderItems === null) {
            $conn = $this->webshop->getConnection();
            $sql = 'SELECT * FROM wp_woocommerce_order_items WHERE order_id = ?';

            /** @noinspection PhpUnhandledExceptionInspection */
            $this->wooOrderItems = $conn->executeQuery($sql, [$this->id])
                ->fetchAllAssociative();
        }
        return $this->wooOrderItems;
    }

    private function getWooOrderItemMeta(?int $id = null): array
    {
        if (is_int($id) && $id > 0) {
            return $this->getWooOrderItemMeta()[$id];
        }

        if ($this->wooOrderItemMeta === null) {
            $conn = $this->webshop->getConnection();
            $sql = sprintf(
                'SELECT order_item_id, meta_key, meta_value FROM wp_woocommerce_order_itemmeta WHERE order_item_id IN(%s)',
                implode(',', array_fill(0, count($this->getWooOrderItems()), '?')),
            );

            /** @noinspection PhpUnhandledExceptionInspection */
            $result = $conn->executeQuery($sql,
                array_map(
                    fn (array $item) => $item['order_item_id'],
                    $this->getWooOrderItems(),
                ),
            )->fetchAllAssociative();

            $this->wooOrderItemMeta = [];
            $orderItemIds = [];

            foreach ($result as $meta) {
                if ($meta['meta_key'] === '_product_id') {
                    $orderItemIds[intval($meta['meta_value'])] = $meta['order_item_id'];
                }

                $this->wooOrderItemMeta[$meta['order_item_id']][$meta['meta_key']] = $meta['meta_value'];
            }

            if ($client = $this->webshop->getGathClient()) {
                $productGateway = new ProductGateway($client);
                $products = $productGateway->findByIds(array_keys($orderItemIds));

                foreach ($products as $product) {
                    $this->wooOrderItemMeta[$orderItemIds[$product->getId()]]['_product_sku'] = $product->getSku();
                }
            }
        }
        return $this->wooOrderItemMeta;
    }

    private function getWooOrderMeta(): array
    {
        if ($this->wooOrderMeta === null) {
            $conn = $this->webshop->getConnection();
            $sql = 'SELECT meta_key, meta_value FROM wp_postmeta WHERE post_id = ?';

            /** @noinspection PhpUnhandledExceptionInspection */
            $this->wooOrderMeta = $conn->executeQuery($sql, [$this->id])
                ->fetchAllKeyValue();
        }
        return $this->wooOrderMeta;
    }

    //</editor-fold>
}
